#!/usr/local/bin/perl
# NCBI の Nucleotide, Nucleotide EST, Protein, データベースから
# gi numbers をダウンロードするスクリプト．

# perl giRetriever.pl

# "Anguilla japinica" を例にしています．以下と同じ数の gi numbers が得られているはずです．
# http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?id=7937

use Bio::DB::EUtilities;
use strict;
use warnings;

#####
my $spName = "Anguilla_japonica";
#####

my $db = "";
my $outDNAgi = "giDNA_".$spName.".txt";
my $outESTgi = "giEST_".$spName.".txt";
my $outAAgi = "giAA_".$spName.".txt";


##### Nucleotide DB を検索
$db = "nucleotide";
my $ref_uidsDNA = &giRetriever($db);
my @uidsDNA = @$ref_uidsDNA;

##### Nucleotide DB を検索
$db = "protein";
my $ref_uidsAA = &giRetriever($db);
my @uidsAA = @$ref_uidsAA;

##### EST DB を検索
$db = "nucest";
my $ref_uidsEST = &giRetriever($db);
my @uidsEST = @$ref_uidsEST;



#### プリントアウト
print "gi numbers for $spName\n";

# Nucleotide DB
print "Nucleotide ".@uidsDNA."\n";
open(DNAOUT,">$outDNAgi");
foreach (@uidsDNA) {
  print DNAOUT "$_\n";
}
close DNAOUT;

## EST DB
print "NucleotideEST ".@uidsEST."\n";
open(ESTOUT,">$outESTgi");
foreach (@uidsEST) {
  print ESTOUT "$_\n";
}
close ESTOUT;

## AA DB
print "Protein ".@uidsAA."\n";
open(AAOUT,">$outAAgi");
foreach (@uidsAA) {
  print AAOUT "$_\n";
}
close AAOUT;

exit;


#### サブルーチン

sub giRetriever{
  my($db) = @_;
  my $factory = Bio::DB::EUtilities->new(-eutil  => 'esearch',
  
                                         -db     => "$db",
                                         # protein,nucleotide,nuccore,nucgss,nucest,structure,genome,gene,journals,
                                         # Bio::Tools::EUtilities::EUtilParameters を参照．
                                         
                                         -term   => "$spName [Organism]",
  #                                       -email  => 'mymail@foo.bar',
                                         -retmax => 10000000);
  my @uids    = $factory->get_ids;
  return (\@uids);
}


# このスクリプトは以下のページを参照に作りました．
# http://search.cpan.org/~sendu/bioperl-1.5.2_102/Bio/DB/EUtilities.pm
# http://www.bioperl.org/wiki/HOWTO:EUtilities_Cookbook#How_do_I_retrieve_the_DNA_sequence_using_EntrezGene_IDs.3F
